/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is autoHideStatusbar.
 *
 * The Initial Developer of the Original Code is
 * Rgis Caspar.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): 
 *    Ihoss
 *    The developers of Adblock
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* 
 * This code is inspired from:
 * http://kb.mozillazine.org/Dev_:_Extensions_:_Example_Code_:_Adding_Drag_and_Drop_to_Statusbarpanel
 * and http://kb.mozillazine.org/Prompt_service
 */


/**
 * autoHideStatusbarDNDObserver constructor
 */
function autoHideStatusbarDNDO()
{
  // nothing
}


/**
 * Initialization function
 */
autoHideStatusbarDNDO.prototype.load = function()
{
  var parent = document.getElementById("status-bar");
  
  if ( !parent ) 
  {
    // no statusbar so we bail...
    return;
  }
  
  var sbi = document.getElementById("autohidestatusbar-statusbar-item");
  var prefVal = AHSPref.statusbaritempos().split(";");
  var sbi2 = document.getElementById(prefVal[1]);

  if (sbi2)
  {
    (prefVal[0] == "after") && ( sbi2 = sbi2.nextSibling );
    parent.insertBefore(sbi, sbi2);
  }
  else
  {
    parent.appendChild(sbi);
  }
}


/**
 * Cleanup function
 */
autoHideStatusbarDNDO.prototype.unload = function()
{
  AHSDNDO = null;
}


/**
 * Return "our" flavor
 */
autoHideStatusbarDNDO.prototype.getSupportedFlavours = function() 
{
  var flavours = new FlavourSet();
  flavours.appendFlavour("autoHideStatusbar-flavor");
  return flavours;
}


/**
 * Called when the user start the drag and drop
 */
autoHideStatusbarDNDO.prototype.onDragStart = function(aEvt, aTransferData, aAction)
{
  var elem = aEvt.target;
  
  while ( elem.id != "autohidestatusbar-statusbar-item" )
  {
    elem = elem.parentNode;
  }
  
  aTransferData.data = new TransferData();
  aTransferData.data.addDataForFlavour("autoHideStatusbar-flavor", elem.id);
  var status = document.getElementById("autohidestatusbar-statusbar-item");
  var statusbar = document.getElementById("status-bar");
  var child = statusbar.firstChild;
  var x = 0, count = 0;
  
  while ( child )
  {
    if (child != status && child.nodeName == "statusbarpanel") 
    {
      // add an id to elements which don't have one
      if (!child.id) 
      {
        ++count;
        
        if ( child.hasAttribute("_id") )
        {
          child.id = child.getAttribute("_id");
        }
        else
        {
          var newId = "statusbarpanel-noID-";
        
          while ( document.getElementById(newId + x) ) 
          {
            ++x;
          }
          
          child.id = newId + x;
          // *** FIXME ***
          // A generated ID id given to any statusbarpanel without one
          // but persist only works for element with ID, so you can't 
          // persist the ID. Therefor, ID for an element can change 
          // between two generation and thus our placement can be 
          // wrong...
          child.setAttribute("persist", "id" + ( (child.persist) ? " " + child.persist : "" ) );
        }
      }
      
      // add event drag and drop event listeners to each statusbarpanel if they don't already have one
      if ( !child.hasAttribute("AHS_drag_ready") )
      {
        child.addEventListener("dragenter", function(event) { nsDragAndDrop.dragEnter(event, AHSDNDO); }, false);
        child.addEventListener("dragover", function(event) { nsDragAndDrop.dragOver(event, AHSDNDO); }, false);
        child.addEventListener("dragdrop", function(event) { nsDragAndDrop.drop(event, AHSDNDO); }, false);
        child.setAttribute("AHS_drag_ready", "true");
      }  
    } 
    
    child = child.nextSibling;
  } 
  
  if ( !AHSPref.statusbaridwarned() && count > 1 )
  {
    // warn the user about possible problem (see above)
    try
    {
      var strBundle = document.getElementById("autoHideStatusbar-strings");
      var check = { value: false };
      var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                              .getService(Components.interfaces.nsIPromptService);
      prompts.alertCheck(window, strBundle.getString("warn.title"), 
                         strBundle.getString("warn.msg"), 
                         strBundle.getString("stopasking"), check);
      AHSPref.setBoolPref("statusbaridwarned", check.value);
    }
    catch(e)
    {
      if (typeof Components.reportError == "function")
      {
        Components.reportError(ex);
      }
      else 
      {
        Components.utils.reportError(ex);
      }
    }
  }
}


/**
 * Called when the user drag over something
 */
autoHideStatusbarDNDO.prototype.onDragOver = function(aEvt, aFlavour, aSession)
{
  // nothing
}


/**
 * Called when the user just finished the drag and drop 
 * Do some cleaning
 */
autoHideStatusbarDNDO.prototype.onDragExit = function(aEvt, aFlavour, aSession)
{
  // nothing
}


/**
 * Called when the user enter something when draging
 */
autoHideStatusbarDNDO.prototype.onDragEnter = function(aEvt, aFlavour, aSession)
{
  // nothing
}


/**
 * Called when the user drop the thing draged
 */
autoHideStatusbarDNDO.prototype.onDrop = function(aEvt, aDropdata, aSession)
{
  if ( aDropdata.data != "" )
  {
    var elem = aEvt.target;
    
    while ( elem.nodeName != "statusbarpanel" )
    {
      elem = elem.parentNode;
    }
    
    // 'delta' is the difference between the mouse position when 
    // the drop event occurs and the middle of the target's boxObject
    // delta > 0 if the mouse is on the right of the middle
    var delta = aEvt.clientX - (elem.boxObject.x + elem.boxObject.width / 2);
    var elem2 = document.getElementById(aDropdata.data);
    var parent = elem.parentNode;
    (delta > 0) && ( elem = elem.nextSibling );
    
    if (elem)
    {
      parent.insertBefore(elem2, elem);
    }
    else
    {
      parent.appendChild(elem2);
    }

    // store the actual position into preferences
    AHSPref.setCharPref("statusbaritempos", ( (delta <= 0) ? "before" : "after" ) + 
                        ";" + elem.id);
  } 
}


var AHSDNDO = new autoHideStatusbarDNDO();

window.addEventListener('load', AHSDNDO.load, false); 
window.addEventListener('unload', AHSDNDO.unload, false);
