/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is autoHideStatusbar.
 *
 * The Initial Developer of the Original Code is
 * Rgis Caspar.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): N/A
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


/**
 * autoHideStatusbarLog constructor
 */
function autoHideStatusbarLog()
{
  this.strBundle = null;
  this.logdoc = null;
  this.log = "";
  this.label = null;
  this.acceptButton = null;
  this.nsIAHSLogService = Components.classes["@caspar.regis.free.fr/ahs;1"]
                                    .getService(Components.interfaces.nsIAHSLogService);
}


/**
 * Initialization function
 */
autoHideStatusbarLog.prototype.load = function()
{
  try
  {
    AHSLog.log = AHSLog.nsIAHSLogService.getFormattedMessages();
  }
  catch(ex)
  {
    AHSLog.reportError(ex);
  }
  
  AHSLog.acceptButton = document.getElementById("autoHideStatusbar-log").getButton("accept");
  AHSLog.logdoc = document.getElementById("logwindow").contentDocument;
  AHSLog.strBundle = document.getElementById("autoHideStatusbar-strings");
  AHSLog.label = document.getElementById("logsize");
  AHSLog.logdoc.body.innerHTML = AHSLog.log;
  AHSLog.label.setAttribute( "value", AHSLog.strBundle.getString("logsize") +
                             " ~" + Math.round(AHSLog.log.length / 1024) + " kb" );
}


/**
 * Cleanup function
 */
autoHideStatusbarLog.prototype.unload = function()
{
  AHSLog = null;
}


/**
 * Cleanup function
 *
 * @param anException: the exception to report
 */
autoHideStatusbarLog.prototype.reportError = function(anException)
{
  if (typeof Components.reportError == "function")
  {
    Components.reportError(anException);
  }
  else 
  {
    Components.utils.reportError(anException);
  }
}


/**
 * Clear the log
 */
autoHideStatusbarLog.prototype.clearLog = function()
{
  try
  {
    this.nsIAHSLogService.clearMessages();
    this.log = this.nsIAHSLogService.getFormattedMessages();
  }
  catch(ex)
  {
    this.reportError(ex);
  }

  this.logdoc.body.innerHTML = this.log;
  this.label.setAttribute( "value", this.strBundle.getString("logsize") + " -" );
  this.acceptButton.disabled = true;
  return true;
}


/**
 * Save the log to a file
 */
autoHideStatusbarLog.prototype.saveLog = function() 
{
  try
  {
    return this.nsIAHSLogService.saveMessages(window, this.strBundle.getString("selectFile"),
                                              "autoHideStatusbar-Log.html");
  } 
  catch(ex)
  {
    this.reportError(ex);
  }
  
  return false;
}


var AHSLog = new autoHideStatusbarLog();

window.addEventListener("load", AHSLog.load, false);
window.addEventListener("unload", AHSLog.unload, false);
