/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is autoHideStatusbar.
 *
 * The Initial Developer of the Original Code is
 * Rgis Caspar.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): 
 *    Philip Chee (SeaMonkey support)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 

/**
 * autoHideStatusbar constructor
 */
function autoHideStatusbar()
{
  this.broadcaster = null;
  this.loading = false;
  this.lastinout = "in";
  this.lastnode = "";
  this.keypressed = false;
  this.waiter = null;
  this.waiter2 = null;
  this.initialized = false;
  this.nsIAHSLogService = null;
  this.nsIObserverService = null;
  this.DLobserver = null;
  this.isMailNews = false;
}


/**
 * Initialization function
 */
autoHideStatusbar.prototype.load = function()
{
  if ( !AHS.initialized )
  {
    AHS.init();
  }
}


/**
 * Cleanup function
 */
autoHideStatusbar.prototype.unload = function()
{
  AHS.log("Uninitialization", 5);  
  AHS.unregister();
  AHSPref = null;
  AHSWPL = null;
  AHS = null;
}


/**
 * Init AHS & Register web progress Listener 
 */
autoHideStatusbar.prototype.register = function() 
{
  this.log("registering progressListener", 4);
  AHSWPL.init();
  window.getBrowser().addProgressListener(AHSWPL, AHSWPL.STATE_START);
}


/**
 * Unregister web progress Listener and DL observer if needed
 */
autoHideStatusbar.prototype.unregister = function() 
{
  if ( this.initialized && !this.isMailNews )
  {
    this.log("unregistering progressListener", 4);
    AHSWPL.destroy();
    window.getBrowser().removeProgressListener(AHSWPL, AHSWPL.STATE_STOP);
    
    if ( this.DLobserver )
    {
      // Download Statusbar is installed
      AHS.log("Download observer destroyed", 4);
      AHS.nsIObserverService.removeObserver(this.DLobserver, "dl-start", false); 
    }
  }
}


/**
 * Add DL observer, log informations and set up statusbar & icons
 */
autoHideStatusbar.prototype.init = function()
{
  var appcontent = document.getElementById("appcontent");
  var mailContent = document.getElementById("mailContent");
  var adbookContent = document.getElementById("addressbookWindow");
  var enabled = AHSPref.enabled();
  
  this.broadcaster = document.getElementById("ahsEnabled");
  this.initialized = true;
  this.isMailNews = ("AHSinMailNews" in window);
  this.broadcaster.setAttribute("ahsEnabled", enabled);
  
  if ( !appcontent && !mailContent && !( adbookContent && this.isMailNews ))
  {
    // we are in "customize toolbar" mode
    window.removeEventListener("keydown", AHS.keydown, false); 
    window.removeEventListener("keyup", AHS.keyup, false); 
    window.removeEventListener("mousemove", AHS.mouse, true);
    window.removeEventListener("mouseover", AHS.TBmouseover, true); 
    window.removeEventListener("mouseout", AHS.mouseout, false); 
    return;
  }
  
  this.log("Initialization", 5);
  var broadcaster2 = document.getElementById("ahsShowSBItem");
  var alwayshide = AHSPref.alwayshide();
  var showstatusbaritem = AHSPref.showstatusbaritem();
  this.broadcaster.setAttribute("moz-collapsed", enabled);
  broadcaster2.setAttribute("moz-collapsed", !showstatusbaritem);
  
  if ( this.isMailNews )
  {
    this.log( "Config: version = " + AHSPref.getVersion() +
              " ; showstatusbaritem = " + showstatusbaritem, 5);
    return;
  }
  
  this.log( "Config: version = " + AHSPref.getVersion() +
            " ; showstatusbaritem = " + AHSPref.showstatusbaritem() +
            " ; alwayshide = " + alwayshide +
            " ; sensitivity = " + AHSPref.sensitivity() + 
            " px / " + AHSPref.hovertime() + 
            " ms ; showonlinkhover = " + AHSPref.showonlinkhover() +
            " (onlyif = " + AHSPref.onlyif() +
            ", timer = " + AHSPref.timer() +
            " ms) ; showonpageload = " + AHSPref.showonpageload(), 5);
  appcontent.addEventListener("mouseover", this.mouseover, false);
  appcontent.addEventListener("resize", function(e) { e.stopPropagation(); }, true);
  
  if ( document.getElementById("downbarHboxTemp") )
  {
    // Download Statusbar is installed
    this.DLobserver = 
    {
      observe: function(aSubject, aTopic, aState)
      {
        AHS.toogle(false);
        AHS.log("Download observer: download started", 3);
      }
    }
    
    try
    {
      if (!this.nsIObserverService)
      {     
        this.nsIObserverService = Components.classes["@mozilla.org/observer-service;1"]
                                            .getService(Components.interfaces.nsIObserverService);
      }
      
      this.nsIObserverService.addObserver(this.DLobserver, "dl-start", false); 
      this.log("'Download Statusbar' detected (download observer added)", 4);        
    }
    catch (ex)
    {
      this.exception(ex);
    }
  }
  
  if (alwayshide && enabled)
  {
    this.log( "toogle (hide statusbar)", 4);
    this.broadcaster.setAttribute("moz-collapsed", true);
  }
  
  if ( !AHS.isMailNews ) 
  {
    AHS.register();
  }
}


/**
 * Toogle the statusbar
 *
 * @param hideit: true if we need to hide the statusbar
 */
autoHideStatusbar.prototype.toogle = function(hideit) 
{
  var alwayshide = (this.isMailNews) ? false : AHSPref.alwayshide();

  if ( !this.broadcaster || !AHSPref.enabled() || alwayshide ||
       this.broadcaster.getAttribute("moz-collapsed") == ("" + hideit) )
  {
    return;
  }
  
  this.log( "toogle (" + ( (hideit) ? "hide" : "show" ) + " statusbar)", 3);
  this.broadcaster.setAttribute("moz-collapsed", hideit);
}


/**
 * Handle keydown event
 *
 * @param aEvt: the event
 */
autoHideStatusbar.prototype.keydown = function(aEvt)
{
  if ( aEvt.keyCode == aEvt.DOM_VK_CONTROL )
  {
    AHS.keypressed = true;
  }
}


/**
 * Handle keyup event
 *
 * @param aEvt: the event
 */
autoHideStatusbar.prototype.keyup = function(aEvt)
{
  if ( aEvt.keyCode == aEvt.DOM_VK_CONTROL )
  {
    AHS.keypressed = false;
  }
}


/**
 * Handle mousemove event and show back the statusbar if we are hovering
 * the "sensitive zone" (Firefox, Flock)
 *
 * @param aEvt: the event
 */
autoHideStatusbar.prototype.mouse = function(aEvt)
{
  var win = document.getElementById("main-window");
  
  if ( AHS.loading || !win )
  {
    return;
  }
  
  if ( win.boxObject.screenY + win.boxObject.height 
       - aEvt.screenY < AHSPref.sensitivity() )
  {
    if ( AHS.lastinout != "in" )
    {
      AHS.log("in the sensitive zone", 3);
      AHS.lastinout = "in";
    }
    
    if (!AHS.waiter2)
    {
      AHS.waiter2 = window.setTimeout( function() 
                                       { 
                                         AHS.toogle(false);
                                         window.setTimeout(function() { AHS.waiter2 = null; }, 100);
                                         AHS.log("timer2 self-destroyed", 2);
                                       }, AHSPref.hovertime() );
      AHS.log("timer2 created", 2);
    }
  } 
  else
  {
    if ( AHS.lastinout != "out" )
    {
      AHS.log("out the sensitive zone", 3);
      AHS.lastinout = "out";
      
      if ( AHS.waiter2 )
      {
        AHS.log("mouseover content", 3); 
        window.clearTimeout(AHS.waiter2);
        AHS.waiter2 = null;
        AHS.log("timer2 destroyed", 2);
      }
    }
    
    // nothing
  }
  
}


/**
 * Handle hovering of the "sensitive zone" (Thunderbird)
 */
autoHideStatusbar.prototype.hover = function()
{
  AHS.log("hbox hovered", 3); 
   
  if (!AHS.waiter2)
  {
    AHS.waiter2 = window.setTimeout( function() 
                                     { 
                                       AHS.toogle(false);
                                       AHS.waiter2 = null;
                                       AHS.log("timer2 self-destroyed", 2);
                                     }, AHSPref.hovertime() );
    AHS.log("timer2 created", 2);
  }
}


/**
 * Handle mouse out of the main window
 *
 * @param aEvt: the event
 */
autoHideStatusbar.prototype.mouseout = function(aEvt)
{
  var win = document.getElementById("main-window");
  
  // Dirty hack to make http://userstyles.org/style/show/993
  // work. When aEvt.target.nodeName is a menu or menupopup
  // The event is somehow destroyed by the top, bottom, left 
  // and right calculations... Bug ??
  if ( !win || aEvt.target.nodeName == "menu" ||
       aEvt.target.nodeName == "menupopup" )
  {
    return;
  }
    
  var top = win.boxObject.screenY;
  var left = win.boxObject.screenX;
  var right = left + win.boxObject.width;
  var bottom = top + win.boxObject.height;
  
  if ( aEvt.screenY > top && aEvt.screenY < bottom &&
       aEvt.screenX > left && aEvt.screenX < right )
  {
    // we are still into the main window => bogus mouseout ?? 
    // cf. http://www.garretwilson.com/notes/computers/internet/browsers/mozilla/mousebug.html
    return;
  }
  
  AHS.log("mouseout mainwin", 3); 
  
  if (AHS.waiter2)
  {
    window.clearTimeout(AHS.waiter2);
    AHS.waiter2 = null;
    AHS.log("timer2 destroyed", 2);
  }
}


/**
 * Handle mouseover event and hide the statusbar if we are not hovering
 * the "sensitive zone" (Thunderbird)
 *
 * @param aEvt: the event
 */
autoHideStatusbar.prototype.TBmouseover = function(aEvt)
{
  if ( !AHS.isMailNews )
  {
    return;
  }
  
  var tg = aEvt.target;
  var nodename = null;
  
  while ( tg && tg.nodeName != "window" && tg.nodeName != "statusbar" )
  {
    try
    {
      tg = tg.parentNode;
    } 
    catch(ex)
    {
      AHS.exception(ex);
      break;
    }
  }
  
  if ( tg )
  {
    nodename = tg.nodeName;
  }
  
  if ( AHS.lastnode != nodename )
  {
    AHS.log("mouseover " + nodename, 3);
    AHS.lastnode = nodename;
  }
  
  AHS.toogle(nodename != "statusbar");
}


/**
 * Handle fullscreen event and add "moz-collapsed" attribute back 
 * to the status-bar.
 *
 * @param aEvt: the event
 */
autoHideStatusbar.prototype.fullScreen = function(aEvt)
{
  var sb = document.getElementById("status-bar");
  var bc = AHS.broadcaster.getAttribute("moz-collapsed");
  AHS.log("onFullscreen", 3);
  window.setTimeout(function(){ sb.setAttribute("moz-collapsed", bc) }, 200);
}


/**
 * Handle mouseover event and show back statusbar if we are hovering a link 
 * (following preferences)
 *
 * @param aEvt: the event
 */
autoHideStatusbar.prototype.mouseover = function(aEvt)
{
  if ( AHS.loading || AHS.waiter2 )
  {
    return;
  }
  
  var nodename = aEvt.target.nodeName;
  var parentnode;
  
  try
  {
    parentnode = aEvt.target.parentNode;
  
    while ( parentnode && parentnode.nodeName != "A" )
    {
      parentnode = parentnode.parentNode;
    }
    
    if ( parentnode && nodename != "A")
    {
      nodename = parentnode.nodeName;
    }
  }
  catch(ex) 
  {
    // do nothing
  }
  
  if ( AHS.lastnode != nodename )
  {
    AHS.log("mouseover " + nodename, 3);
    AHS.lastnode = nodename;
  }
    
  if ( AHSPref.showonlinkhover() )
  {
    switch( AHSPref.onlyif() ) 
    {
      case 1: // show always
        // nothing
        break;
      
      case 2: // show on hovering with CTRL pressed
        if ( !AHS.keypressed )
        {
          AHS.log("keypressed = false", 1);
          
          if (nodename == "A")
          {
            return;
          }
        } 
        else 
        {
          AHS.log("keypressed = true", 1);
        }
        break;
      
      case 3: // show on hovering for more than xxx sec.
        AHS.log("waiter = " + ( autoHideStatusbar.waiter || "null" ), 1);
               
        if ( AHS.waiter )
        {
          window.clearTimeout(AHS.waiter);
          AHS.waiter = null;
          AHS.log("timer destroyed", 2);
        } 
        else if ( nodename == "A" )
        {
          AHS.waiter = window.setTimeout( function() 
                                          { 
                                            AHS.toogle(false);
                                            AHS.waiter = null;
                                            AHS.log("timer self-destroyed", 2);
                                          }, AHSPref.timer() );
          AHS.log("timer created", 2);
        }
        
        AHS.toogle(true);
        return;
      
      default: // fallback
        // nothing
    }
  } 
  else
  {
    if ( nodename == "A" )
    {
      return;
    }
  }

  AHS.toogle(nodename != "A");
}


/**
 * Send exceptions to both JS console and the log
 *
 * @param anException: the exception
 */
autoHideStatusbar.prototype.exception = function(anException)
{
  AHS.log(anException.toString(), 4);
  
  if (typeof Components.reportError == "function")
  {
    Components.reportError(anException);
  }
  else 
  {
    Components.utils.reportError(anException);
  }
}


/**
 * Set the tooltip text for statusbar and toolbar icons
 *
 * @param aTooltip: the tooltip to set
 */
autoHideStatusbar.prototype.setTooltip = function(aTooltip)
{
  var strBundle = document.getElementById("autoHideStatusbar-strings");
  aTooltip.setAttribute( "label", "autoHideStatusbar " + 
                         strBundle.getString("is" + 
                         ( AHSPref.enabled() ? "En" : "Dis" ) +
                         "abled") );
}


/**
 * Add a message to the log
 *
 * @param aMessage: message text 
 * @param aPriority: message priority
 */
autoHideStatusbar.prototype.log = function(aMessage, aPriority) 
{  
  if ( AHSPref.debug() && AHSPref.enabled() ) 
  {
    if (!this.nsIAHSLogService)
    {
      this.nsIAHSLogService = Components.classes["@caspar.regis.free.fr/ahs;1"]
                                        .getService(Components.interfaces.nsIAHSLogService);
    }
    
    this.nsIAHSLogService.addMessage(aMessage, aPriority);
  }
}


/**
 * Handle changes made by toolbar/statusbar icon click
 *
 * @param aEvt: the click event
 */
autoHideStatusbar.prototype.toolbarClick = function(aEvt)
{
  switch ( aEvt.button )
  {
    case 0: // left-click
      AHSPref.setBoolPref("enabled", !AHSPref.enabled());
      break;
      
    case 1: // middle-click
      // nothing
      break;
      
    case 2: // right-click
      openDialog("chrome://autohidestatusbar/content/prefs.xul", 
                 "", "chrome,titlebar,toolbar,centerscreen,modal",
                 AHS.isMailNews); 
    
    default: // fallback
      // nothing
  }
  
  aEvt.preventDefault();
}


var AHS = new autoHideStatusbar();

window.addEventListener("load", AHS.load, false);
window.addEventListener("unload", AHS.unload, false);
window.addEventListener("keydown", AHS.keydown, false); 
window.addEventListener("keyup", AHS.keyup, false); 
window.addEventListener("mousemove", AHS.mouse, true);
window.addEventListener("mouseover", AHS.TBmouseover, true); 
window.addEventListener("mouseout", AHS.mouseout, false); 
window.addEventListener("fullscreen", AHS.fullScreen, false);
