/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is autoHideStatusbar.
 *
 * The Initial Developer of the Original Code is
 * Rgis Caspar.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): 
 *    Philip Chee (SeaMonkey support)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


/**
 * AHSprefMan constructor
 */
function AHSprefMan()
{
  this.prefs = null;
  this.init();
}


/**
 * AHSprefMan initialization
 */
AHSprefMan.prototype.init = function()
{
  var nsIPrefService = Components.classes["@mozilla.org/preferences-service;1"]
                                 .getService(Components.interfaces.nsIPrefService);
  this.prefs = nsIPrefService.getBranch("extensions.autohidestatusbar.");
}


/**
 * Return specified pref. or a default value if a problem occurs
 *
 * @param name: pref. name
 * @param defVal: default value
 */
AHSprefMan.prototype.pref = function(name, defVal) 
{
  try 
  {
    var result;
    
    switch ( this.prefs.getPrefType(name) )
    {
      case this.prefs.PREF_INT:
        result = this.prefs.getIntPref(name);
        break;
      
      case this.prefs.PREF_BOOL:
        result = this.prefs.getBoolPref(name);
        break;
      
      case this.prefs.PREF_STRING:
        result = this.prefs.getCharPref(name);
        break;
        
      case this.prefs.PREF_INVALID:
      default: 
        result = defVal
    }
    
    return result;
  } 
  catch (ex) 
  {
    if (typeof Components.reportError == "function")
    {
      Components.reportError(ex);
    }
    else 
    {
      Components.utils.reportError(ex);
    }
    
    return defVal;
  }
}


/**
 * Set specified bool pref. 
 *
 * @param name: pref. name
 * @param val: new pref. value
 */
AHSprefMan.prototype.setBoolPref = function(name, val) 
{
  try 
  {
    this.prefs.setBoolPref(name, val);
  } 
  catch (ex) 
  {
    AHS.exception(ex);
  }
}


/**
 * Set specified char pref. 
 *
 * @param name: pref. name
 * @param val: new pref. value
 */
AHSprefMan.prototype.setCharPref = function(name, val) 
{
  try 
  {
    this.prefs.setCharPref(name, val);
  } 
  catch (ex) 
  {
    AHS.exception(ex);
  }
}


/**
 * Return extension version
 */
AHSprefMan.prototype.getVersion = function()
{
  if (!("nsIExtensionManager" in Components.interfaces) ||
      !("@mozilla.org/extensions/manager;1" in Components.classes))
  {
    return "Unknow";
  }
  
  try
  {
    const AHS_ID = "{ee56ecf0-6e7a-479a-8162-e123a991c7e7}";
    var nsIExtensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
                                        .getService(Components.interfaces.nsIExtensionManager);
    return nsIExtensionManager.getItemForID(AHS_ID).version;
  }
  catch(ex)
  {
    AHS.exception(ex);
    return "Unknow";
  }
}


/**
 * Test if current app. is Thunderbird and return true if so
 */
AHSprefMan.prototype.isThunderbird = function()
{
  const THUNDERBIRD_ID = "{3550f703-e582-4d05-9a08-453d09bdfdc6}"; 
  var nsIXULAppInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                                .getService(Components.interfaces.nsIXULAppInfo);
  return (nsIXULAppInfo.ID == THUNDERBIRD_ID);
}


/**
 * Wrappers around pref() for quick access
 */
AHSprefMan.prototype.sensitivity = function() { return this.pref("sensitivity", 5); }
AHSprefMan.prototype.timer = function() { return this.pref("timer", 1000); }
AHSprefMan.prototype.onlyif = function() { return this.pref("onlyif", 1); }
AHSprefMan.prototype.debug = function() { return this.pref("debug", false); }
AHSprefMan.prototype.showonlinkhover = function() { return this.pref("showonlinkhover", true); }
AHSprefMan.prototype.showstatusbaritem = function() { return this.pref("showstatusbaritem", true); }
AHSprefMan.prototype.statusbaritempos = function() { return this.pref("statusbaritempos", "statusbar-resizerpanel"); }
AHSprefMan.prototype.statusbaridwarned = function() { return this.pref("statusbaridwarned", false); }
AHSprefMan.prototype.showonpageload = function() { return this.pref("showonpageload", true); }
AHSprefMan.prototype.enabled = function() { return this.pref("enabled", true); }
AHSprefMan.prototype.alwayshide = function() { return this.pref("alwayshide", false); }
AHSprefMan.prototype.hovertime = function() { return this.pref("hovertime", false); }


var AHSPref = new AHSprefMan();
