/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is autoHideStatusbar.
 *
 * The Initial Developer of the Original Code is
 * Rgis Caspar.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): 
 *    Asqueella
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* Code taken from http://kb.mozillazine.org/Dev_:_Using_preferences#Using_preferences_observers */


/**
 * AHSPrefObserver constructor
 */
function AHSPrefObserver()
{
  this.branch = null;
}


/**
 * Register the pref observer
 */
AHSPrefObserver.prototype.register = function()
{
  var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefService);
  AHSPO.branch = prefService.getBranch("extensions.autohidestatusbar.");
  AHSPO.branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  AHSPO.branch.addObserver("", AHSPO, false);
}


/**
 * Unregister the pref observer and do some cleanup
 */
AHSPrefObserver.prototype.unregister = function()
{
  if (!AHSPO.branch)
  {
    return;
  }
    
  AHSPO.branch.removeObserver("", AHSPO);
  AHSPO = null;
}


/**
 * Observation function - called when a pref change
 *
 * @param aSubject: the nsIPrefBranch we're observing (after appropriate QI)
 * @param aTopic: a string describing what's changed (only "nsPref:changed" is handled here)
 * @param aData: the name of the pref that's been changed (relative to aSubject)
 */
AHSPrefObserver.prototype.observe = function(aSubject, aTopic, aData)
{
  if (aTopic != "nsPref:changed")
  {
    return;
  }
  
  switch (aData) 
  {
    case "enabled": // extensions.autohidestatusbar.enabled was changed
      var enabled = this.branch.getBoolPref(aData);
      var broadcaster = document.getElementById("ahsEnabled");
      
      if (broadcaster)
      {
        broadcaster.setAttribute("ahsEnabled", enabled);
        broadcaster.setAttribute("moz-collapsed", enabled);
      }
      
      break;
    
    case "showstatusbaritem": // extensions.autohidestatusbar.showstatusbaritem was changed
      var showstatusbaritem = this.branch.getBoolPref(aData);
      var broadcaster = document.getElementById("ahsShowSBItem");
      
      if (broadcaster)
      {
        broadcaster.setAttribute("moz-collapsed", !showstatusbaritem);
      }
      
      break;
    
    default:
      // nothing
  }
}


var AHSPO = new AHSPrefObserver();

window.addEventListener("load", AHSPO.register, false);
window.addEventListener("unload", AHSPO.unregister, false);
