/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is autoHideStatusbar.
 *
 * The Initial Developer of the Original Code is
 * Rgis Caspar.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): N/A
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


/**
 * autoHideStatusbarPrefs constructor
 */
function autoHideStatusbarPrefs()
{
  this.isMailNews = false;
}


/**
 * Initialization function
 */
autoHideStatusbarPrefs.prototype.load = function()
{
  if (window.arguments && window.arguments[0] != null)
  {
    // pref dialog comes from right-click on icon
    AHSPrefs.isMailNews = window.arguments[0];
  } 
  else
  {   
    // pref dialog comes from Add-Ons Manager
    AHSPrefs.isMailNews = AHSPref.isThunderbird();
  }
  
  AHSPrefs.enabledChanged();
  AHSPrefs.debugChanged();
}


/**
 * Cleanup function
 */
autoHideStatusbarPrefs.prototype.unload = function()
{
  AHSPrefs = null;
}


/**
 * Namespace resolver for XPath
 *
 * @param aNS: a namespace
 */
autoHideStatusbarPrefs.prototype.NSResolver = function(aNS) 
{ 
  return ( ( aNS == "xul" ) ? 
           "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul" :
           null );
}


/**
 * Handle "debug" pref changes
 */
autoHideStatusbarPrefs.prototype.debugChanged = function() 
{
  var debug = document.getElementById("extensions.autohidestatusbar.debug").value;
  var logButton = document.getElementById("autoHideStatusbar-options").getButton("extra2");
  logButton.disabled = !debug;
}


/**
 * Handle "enabled" pref changes
 */
autoHideStatusbarPrefs.prototype.enabledChanged = function() 
{
  var enabled = document.getElementById("extensions.autohidestatusbar.enabled").value;
  var alwayshide = document.getElementById("extensions.autohidestatusbar.alwayshide").value;
  var image = document.getElementById("showstatusbaritem-image");
  
  if ( !this.isMailNews )
  {
    var res = document.evaluate("//xul:groupbox//xul:checkbox[@id!='showstatusbaritem'] | " + 
                                "//xul:groupbox//xul:spinner | " +
                                "//xul:groupbox//xul:radiogroup | " +
                                "//xul:groupbox//xul:label", 
                                document, 
                                this.NSResolver,
                                XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
                                null);
    var elm;
    
    for (var i = 0; elm = res.snapshotItem(i); i++) 
    {
      elm.disabled = !enabled;
    }
  } 
  else
  {
    var cbox = document.getElementById("alwayshide");
    var cbox2 = document.getElementById("debug");
    var hbox = document.getElementById("autoHideStatusbar-hbox-sensitivity");
    var gbox2 = document.getElementById("autoHideStatusbar-gbox2");
    
    if (cbox && hbox && gbox2)
    {
      cbox.parentNode.removeChild(cbox);
      hbox.parentNode.removeChild(hbox);
      gbox2.parentNode.removeChild(gbox2);
      window.sizeToContent();
    }
    
    cbox2.disabled = !enabled;
  }
  
  if ( !enabled && alwayshide )
  {
    var ahb = document.getElementById("showstatusbaritem");
    ahb.disabled = true;
  }
  
  if ( enabled && !this.isMailNews )
  {
    this.alwayshideChanged();
  }
  
  image.setAttribute("ahsEnabled", enabled);
}


/**
 * Handle "alwayshide" pref changes
 */
autoHideStatusbarPrefs.prototype.alwayshideChanged = function()
{
  var alwayshide = document.getElementById("extensions.autohidestatusbar.alwayshide").value;
  var res = document.evaluate("//xul:groupbox//xul:checkbox[@id!='enable' and " + 
                              "@id!='alwayshide' and @id!='debug'] | " +
                              "//xul:groupbox//xul:spinner | " +
                              "//xul:groupbox//xul:radiogroup | " +
                              "//xul:groupbox//xul:label", 
                              document, 
                              this.NSResolver,
                              XPathResult.UNORDERED_NODE_SNAPSHOT_TYPE,
                              null);
  var elm;
  
  for (var i = 0; elm = res.snapshotItem(i); i++) 
  {
    elm.disabled = alwayshide;
  }
  
  if (!alwayshide)
  {
    this.onlyifChanged();
    this.showonlinkhoverChanged();
  }
}


/**
 * Handle "showonlinkhover" pref changes
 */
autoHideStatusbarPrefs.prototype.showonlinkhoverChanged = function()
{
  var showonlinkhover = document.getElementById("extensions.autohidestatusbar.showonlinkhover").value;
  var onlyif = document.getElementById("extensions.autohidestatusbar.onlyif");
  var timer = document.getElementById("timer");
  var msec = document.getElementById("msec");
  onlyif.disabled = !showonlinkhover;
  timer.disabled = !showonlinkhover;
  msec.disabled = !showonlinkhover;
  
  if (showonlinkhover)
  {
    this.onlyifChanged();
  }
  
}


/**
 * Handle "onlyif" pref changes
 */
autoHideStatusbarPrefs.prototype.onlyifChanged = function()
{
  var onlyif = document.getElementById("extensions.autohidestatusbar.onlyif").value;
  var timer = document.getElementById("timer");
  timer.disabled = (onlyif != 3);
}


var AHSPrefs = new autoHideStatusbarPrefs();

window.addEventListener("load", AHSPrefs.load, false);
window.addEventListener("unload", AHSPrefs.unload, false);
